//This snippet of code shows you how to parse log file (in the common log format)

//Format of the incoming raw record
RawLayout := RECORD
  STRING rawTxt;
end;
 
//A sample of a data file that is inlined 
fileRaw := DATASET([{'127.0.0.1 - frank [10/Oct/2000:13:55:36 -0700] "GET /apache_pb.gif HTTP/1.0" 200 2326'},
                    {'10.0.0.1 - george [10/Oct/2000:01:55:36 -0700] "GET /top.gif HTTP/1.0" 200 2326'}], RawLayout);
 
//Regular expression patterns to parse parts of each line
PATTERN alphaFmt := PATTERN('[a-zA-Z]')+;
PATTERN alphaNumbFmt := PATTERN('[a-zA-Z0-9]')+;
PATTERN sepFmt :=  ' '+;
PATTERN numFmt := PATTERN('[0-9]')+;
PATTERN ipFmt := numFmt '.' numFmt '.' numFmt '.' numFmt;
PATTERN identFmt := '-';
PATTERN authuserFmt := alphaNumbFmt;
PATTERN hoursFromGMT := PATTERN('[\\-\\+]') numFmt;
PATTERN yearFmt := numFmt;
PATTERN monthFmt := alphaNumbFmt;
PATTERN dayFmt := numFmt;
PATTERN hoursFmt := numFmt;
PATTERN minutesFmt := numFmt;
PATTERN secondsFmt := numFmt;
PATTERN dateFmt := '[' dayFmt '/' monthFmt '/' yearFmt ':' hoursFmt ':' minutesFmt ':' secondsFmt ' ' hoursFromGMT ']';
PATTERN cmdFmt := alphaFmt;
PATTERN notQuoteFmt := PATTERN('[^"]')*;
PATTERN paramsFmt := opt('?' notQuoteFmt);
PATTERN urlFmt := PATTERN('[^"\\?]')*;
PATTERN httpMethodFmt := 'HTTP/' numFmt '.' numFmt;
PATTERN requestFmt := '"' cmdFmt urlFmt paramsFmt httpMethodFmt '"';
PATTERN statusFmt := numFmt;
PATTERN bytesFmt := numFmt;

//Tokens in a line  
PATTERN line :=  ipFmt sepFmt identFmt sepFmt authUserFmt sepFmt dateFmt sepFmt requestFmt sepFmt statusFmt sepFmt bytesFmt;
 
//Input record layout for the extracted data 
LogLayout := RECORD
   STRING ip := MATCHTEXT(ipFmt);
   STRING authUser := MATCHTEXT(authuserFmt);
   STRING date := MATCHTEXT(dateFmt);
   STRING request := MATCHTEXT(requestFmt);
   STRING status := MATCHTEXT(statusFmt);
   STRING bytes := MATCHTEXT(bytesFmt);
end;

//Parse the file 
logFile := PARSE(fileRaw,
                 rawTxt,
                 line,
                 LogLayout,first);
 
 
ErrorLayout := RECORD
  STRING t := fileRaw.rawTxt;
end;
  
e := PARSE(fileRaw,
           rawTxt,
           line,
           ErrorLayout,NOT MATCHED ONLY);
                                 
OUTPUT(logFile);                                                          
OUTPUT(e);
